@@ -21,13 +21,13 @@
 
 #include "config.h"
 
-#define DEFAULT_MEM_CARD_1 "/opt/maemo/usr/games/.pcsx/memcards/card1.mcd"
-#define DEFAULT_MEM_CARD_2 "/opt/maemo/usr/games/.pcsx/memcards/card2.mcd"
+#define DEFAULT_MEM_CARD_1 "/.pcsx/memcards/card1.mcd"
+#define DEFAULT_MEM_CARD_2 "/.pcsx/memcards/card2.mcd"
 #define MEMCARD_DIR "/.pcsx/memcards/"
 #define PLUGINS_DIR "/.pcsx/plugins/"
 #define PLUGINS_CFG_DIR "/.pcsx/plugins/cfg/"
 #define PCSX_DOT_DIR "/.pcsx/"
-#define STATES_DIR "/opt/maemo/usr/games/.pcsx/sstates/"
+#define STATES_DIR "/.pcsx/sstates/"
 #define CHEATS_DIR "/.pcsx/cheats/"
 #define PATCHES_DIR "/.pcsx/patches/"
 #define BIOS_DIR "/bios/"
