@@ -74,7 +74,7 @@ static inline void pcnt_print(float fps)
 static inline unsigned int pcnt_get(void)
 {
 	unsigned int val;
-#ifdef __ARM_ARCH_7A__
+#if defined(__ARM_ARCH_7A__) && !defined(MAEMO)
 	__asm__ volatile("mrc p15, 0, %0, c9, c13, 0"
 			 : "=r"(val));
 #else
