@@ -102,7 +102,7 @@ window_key_proxy(GtkWidget *widget,
 				emu_save_state(state_slot);
 				
 				char buf[MAXPATHLEN];
-				sprintf (buf,"screenshots/%s.%3.3d",file_name,state_slot);
+				sprintf (buf,"/opt/maemo/usr/games/screenshots/%s.%3.3d",file_name,state_slot);
 				writepng(buf, image->mem, pl_buf_w,pl_buf_h);
 
 			return;
