@@ -33,10 +33,10 @@ extern int iUseInterpolation;
 extern int iXAPitch;
 extern int iSPUIRQWait;
 extern int iUseTimer;
-extern char *cfile;
 extern int g_opts;
 int gles=0;
-
+char * file_name;
+  
 static void ChangeWorkingDirectory(char *exe)
 {
 	s8 exepath[1024];
@@ -63,7 +63,7 @@ int maemo_main(int argc, char **argv)
 	strcpy(Config.PluginsDir, "/opt/maemo/usr/games/plugins");
 	snprintf(Config.PatchesDir, sizeof(Config.PatchesDir), "/opt/maemo/usr/games" PATCHES_DIR);
 	Config.PsxAuto = 1;
-	
+	char* screen;
 	// read command line options
 	for (i = 1; i < argc; i++) {
 		     if (!strcmp(argv[i], "-psxout")) Config.PsxOut = 1;
@@ -139,10 +139,11 @@ int maemo_main(int argc, char **argv)
 		}
 	}
 	
-	cfile=malloc(sizeof(char[255]));
-	sprintf (cfile, "%s.save.%s", file,loadst);
 
 	hildon_init(&argc, &argv);
+	
+file_name=strrchr(cdfile,'/');
+file_name++;
 
 	
 	if (cdfile)
@@ -156,7 +157,7 @@ int maemo_main(int argc, char **argv)
 		return 1;
 	}
 
-
+	pcnt_hook_plugins();
 	if (OpenPlugins() == -1) {
 		return 1;
 	}
@@ -182,7 +183,7 @@ int maemo_main(int argc, char **argv)
 	// If a state has been specified, then load that
 	if (loadst) {
 		char state_filename[MAXPATHLEN];
-		int ret = get_state_filename(state_filename, sizeof(state_filename), loadst - 1);
+		int ret = get_state_filename(state_filename, sizeof(state_filename), loadst-1);
 		if (ret == 0)
 			ret = LoadState(state_filename);
 		printf("%s state %s\n", ret ? "failed to load" : "loaded", state_filename);
